@@ -1,6 +1,7 @@
 package kong
 
 import (
+	"encoding"
 	"encoding/json"
 	"io/ioutil"
 	"math/bits"
@@ -15,8 +16,10 @@ import (
 )
 
 var (
-	mapperValueType = reflect.TypeOf((*MapperValue)(nil)).Elem()
-	boolMapperType  = reflect.TypeOf((*BoolMapper)(nil)).Elem()
+	mapperValueType       = reflect.TypeOf((*MapperValue)(nil)).Elem()
+	boolMapperType        = reflect.TypeOf((*BoolMapper)(nil)).Elem()
+	textUnmarshalerType   = reflect.TypeOf((*encoding.TextUnmarshaler)(nil)).Elem()
+	binaryUnmarshalerType = reflect.TypeOf((*encoding.BinaryUnmarshaler)(nil)).Elem()
 )
 
 // DecodeContext is passed to a Mapper's Decode().
@@ -57,6 +60,34 @@ func (m *mapperValueAdapter) IsBool() bool {
 	return m.isBool
 }
 
+type textUnmarshalerAdapter struct{}
+
+func (m *textUnmarshalerAdapter) Decode(ctx *DecodeContext, target reflect.Value) error {
+	var value string
+	err := ctx.Scan.PopValueInto("value", &value)
+	if err != nil {
+		return err
+	}
+	if target.Type().Implements(textUnmarshalerType) {
+		return target.Interface().(encoding.TextUnmarshaler).UnmarshalText([]byte(value))
+	}
+	return target.Addr().Interface().(encoding.TextUnmarshaler).UnmarshalText([]byte(value))
+}
+
+type binaryUnmarshalerAdapter struct{}
+
+func (m *binaryUnmarshalerAdapter) Decode(ctx *DecodeContext, target reflect.Value) error {
+	var value string
+	err := ctx.Scan.PopValueInto("value", &value)
+	if err != nil {
+		return err
+	}
+	if target.Type().Implements(textUnmarshalerType) {
+		return target.Interface().(encoding.BinaryUnmarshaler).UnmarshalBinary([]byte(value))
+	}
+	return target.Addr().Interface().(encoding.BinaryUnmarshaler).UnmarshalBinary([]byte(value))
+}
+
 // A Mapper represents how a field is mapped from command-line values to Go.
 //
 // Mappers can be associated with concrete fields via pointer, reflect.Type, reflect.Kind, or via a "type" tag.
@@ -137,11 +168,22 @@ func (r *Registry) ForType(typ reflect.Type) Mapper {
 			return &mapperValueAdapter{impl.Implements(boolMapperType)}
 		}
 	}
+	// Next, try explicitly registered types.
 	var mapper Mapper
 	var ok bool
 	if mapper, ok = r.types[typ]; ok {
 		return mapper
-	} else if mapper, ok = r.kinds[typ.Kind()]; ok {
+	}
+	// Next try stdlib unmarshaler interfaces.
+	for _, impl := range []reflect.Type{typ, reflect.PtrTo(typ)} {
+		if impl.Implements(textUnmarshalerType) {
+			return &textUnmarshalerAdapter{}
+		} else if impl.Implements(binaryUnmarshalerType) {
+			return &binaryUnmarshalerAdapter{}
+		}
+	}
+	// Finally try registered kinds.
+	if mapper, ok = r.kinds[typ.Kind()]; ok {
 		return mapper
 	}
 	return nil
@@ -207,7 +249,8 @@ func (r *Registry) RegisterDefaults() *Registry {
 		RegisterType(reflect.TypeOf(&url.URL{}), urlMapper()).
 		RegisterName("path", pathMapper(r)).
 		RegisterName("existingfile", existingFileMapper(r)).
-		RegisterName("existingdir", existingDirMapper(r))
+		RegisterName("existingdir", existingDirMapper(r)).
+		RegisterName("counter", counterMapper())
 }
 
 type boolMapper struct{}
@@ -217,7 +260,17 @@ func (boolMapper) Decode(ctx *DecodeContext, target reflect.Value) error {
 		token := ctx.Scan.Pop()
 		switch v := token.Value.(type) {
 		case string:
-			target.SetBool(strings.ToLower(v) == "true")
+			v = strings.ToLower(v)
+			switch v {
+			case "true", "1", "yes":
+				target.SetBool(true)
+
+			case "false", "0", "no":
+				target.SetBool(false)
+
+			default:
+				return errors.Errorf("bool value must be true, 1, yes, false, 0 or no but got %q", v)
+			}
 
 		case bool:
 			target.SetBool(v)
@@ -356,6 +409,7 @@ func mapDecoder(r *Registry) MapperFunc {
 			target.Set(reflect.MakeMap(target.Type()))
 		}
 		el := target.Type()
+		sep := ctx.Value.Tag.MapSep
 		var childScanner *Scanner
 		if ctx.Value.Flag != nil {
 			t := ctx.Scan.Pop()
@@ -365,7 +419,7 @@ func mapDecoder(r *Registry) MapperFunc {
 			}
 			switch v := t.Value.(type) {
 			case string:
-				childScanner = Scan(SplitEscaped(v, ';')...)
+				childScanner = Scan(SplitEscaped(v, sep)...)
 
 			case []map[string]interface{}:
 				for _, m := range v {
@@ -540,6 +594,25 @@ func existingDirMapper(r *Registry) MapperFunc {
 	}
 }
 
+func counterMapper() MapperFunc {
+	return func(ctx *DecodeContext, target reflect.Value) error {
+		switch target.Kind() {
+		case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
+			target.SetInt(target.Int() + 1)
+
+		case reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64:
+			target.SetUint(target.Uint() + 1)
+
+		case reflect.Float32, reflect.Float64:
+			target.SetFloat(target.Float() + 1)
+
+		default:
+			return errors.Errorf("type:\"counter\" must be used with a numeric field")
+		}
+		return nil
+	}
+}
+
 func urlMapper() MapperFunc {
 	return func(ctx *DecodeContext, target reflect.Value) error {
 		var urlStr string
